@@ -24,6 +24,7 @@ from .PayloadFilterKeys import rapuiServerEcho
 from .PayloadIO import PayloadIO
 from .PayloadPriority import DEFAULT_PRIORITY
 from .VortexABC import VortexABC, VortexInfo
+from .VortexUtil import logLargeMessages
 
 logger = logging.getLogger(__name__)
 
@@ -31,8 +32,8 @@ logger = logging.getLogger(__name__)
 VortexServer
 """
 
-HEART_BEAT_PERIOD = 5.0
-HEART_BEAT_TIMEOUT = 35.0
+HEART_BEAT_PERIOD = 10.0
+HEART_BEAT_TIMEOUT = 180.0
 
 
 class VortexServer(VortexABC):
@@ -40,6 +41,8 @@ class VortexServer(VortexABC):
     The static instance of the controller
     """
 
+    _DEBUG_LOGGING = False
+
     def __init__(self, name: str, requiresBase64Encoding=True) -> None:
         # Simple initialisations up the top
         self._name = name
@@ -110,7 +113,8 @@ class VortexServer(VortexABC):
         return "%s:%s" % (connection.ip, connection.port)
 
     def connectionOpened(self, httpSession, vortexConnection):
-        # print "VortexServer - connectionOpened"
+        if self._DEBUG_LOGGING:
+            logger.debug("VortexServer - connectionOpened")
 
         vortexUuid = vortexConnection.remoteVortexUuid
         assert vortexUuid
@@ -135,7 +139,8 @@ class VortexServer(VortexABC):
         self._connectionByVortexUuid[vortexUuid] = vortexConnection
 
     def connectionClosed(self, conn):
-        # print "VortexServer - connectionClosed"
+        if self._DEBUG_LOGGING:
+            logger.debug("VortexServer - connectionClosed")
 
         vortexUuid = conn.remoteVortexUuid
 
@@ -211,6 +216,8 @@ class VortexServer(VortexABC):
         if vortexMsgs is None:
             vortexMsgs = [PayloadEnvelope().toVortexMsg()]
 
+        logLargeMessages(logger, vortexMsgs, vortexUuid)
+
         if isMainThread():
             return self._sendVortexMsgLater(
                 vortexMsgs, vortexUuid=vortexUuid, priority=priority
